;=======================================================================
;    - 24
;-----------------------------------------------------------------------
; P1: LED DISPLAY, 4 BUTTONS & SWITCH
; BANK0 & TIMER0 are using by rtc 
;=======================================================================
;include "hex.inc"
include "8051x.inc"
include "max7219.inc"
include "listing.inc"
;=======================================================================
LED  equ P3.7
BUT  equ P2.7
;-----------------------------------------------------------------------
SW   equ SPI_DIN
;-----------------------------------------------------------------------
BUT0 equ 1; P1.0
BUT1 equ 2; P1.1
BUT2 equ 4; P1.2
BUT3 equ 8; P1.3
;-----------------------------------------------------------------------
S_TURBO equ SW
;-----------------------------------------------------------------------
DIG_NUM=8
DATA NUMS,DIG_NUM
DATA KEYS
;-----------------------------------------------------------------------
DATA TX1 ; additional byte for timer 1
;-----------------------------------------------------------------------
FLAG TEN_MS
FLAG RUN
FLAG BLANK_ZERO
;-----------------------------------------------------------------------
RAM_START = 2000h
;-----------------------------------------------------------------------
    org RAM_START
;-----------------------------------------------------------------------
    jmp start
;-----------------------------------------------------------------------
;    orgx 0003h+RAM_START,-1 ; INT0
;    reti
;-----------------------------------------------------------------------
    orgx 000Bh+RAM_START,-1 ; TF0
    jmp scan
;-----------------------------------------------------------------------
;    orgx 0013h+RAM_START,-1 ; INT1
;    reti
;-----------------------------------------------------------------------
    orgx 001Bh+RAM_START,-1 ; TF1
    inc TX1
    reti    
;-----------------------------------------------------------------------
;    orgx 0023h+RAM_START,-1 ; TI & RI
;    reti
;-----------------------------------------------------------------------
;    orgx 002Bh+RAM_START,-1 ; TF2 & EXF2
;    reti
;=======================================================================
include "spi.inc"
;=======================================================================
;
;=======================================================================
num2led: ; A = 0..15 -> A = 7 segment LED combination
    anl  A,#0Fh
    inc  A
    movc A,@A+PC
    ret
    db DIG_0,DIG_1,DIG_2,DIG_3,DIG_4,DIG_5,DIG_6,DIG_7    
    db DIG_8,DIG_9,DIG_A,DIG_B,DIG_C,DIG_D,DIG_E,DIG_F    
;-----------------------------------------------------------------------
show_sym: ; A - symbol, B - place (0..7)
    xch  A,B 
    add  A,#MAX7219_DIGIT0
    jmp  spi16_send
;-----------------------------------------------------------------------
show_row: ; R0->row start (leftmost symbol); R0,R1,A,B - destroyed
    mov  R1,#DIG_NUM
.loop:
    mov  B,R1
    dec  B
    mov  A,@R0
    call show_sym
    inc  R0
    djnz R1,.loop
    ret    
;-----------------------------------------------------------------------
cls: ; R0 destroyed
    push ACC
    push B
    mov  R0,#MAX7219_DIGIT0
.loop:
    clr  B
    mov  A,R0
    call spi16_send
    inc  R0
    cjne R0,#MAX7219_DIGIT7+1,.loop
    pop  B
    pop  ACC
    ret
;-----------------------------------------------------------------------
screen_init:
    mov  DPTR,#.data
.loop:
    clr  A
    movc A,@A+DPTR
    inc  DPTR
    jz   cls
    mov  B,A
    clr  A
    movc A,@A+DPTR
    inc  DPTR
    xch  A,B
    call spi16_send
    jmp  .loop
.data:
    db MAX7219_TEST,0
    db MAX7219_DECODEMODE,0
    db MAX7219_SCANLIMIT,7
    db MAX7219_INTENSITY,3
    db MAX7219_SHUTDOWN,1
    db 0
;=======================================================================
;
;=======================================================================
;   1 ms : 2000 ticks @ 24 MHz : 24000000 / ( 12 * 1000 )
;-----------------------------------------------------------------------
ONE_MS = -2000 + 11 ; 11 ticks between stop and start timer 0 again
;-----------------------------------------------------------------------
;   R3 - ms counter
;-----------------------------------------------------------------------
scan:
;-----------------------------------------------------------------------
    clr  TR0
;-----------------------------------------------------------------------
    push PSW                     ; 2
    push ACC                     ; 2
;-----------------------------------------------------------------------
    mov  A,TL0                   ; 1
    add  A,#255 and ONE_MS       ; 1
    mov  TL0,A                   ; 1
    mov  A,TH0                   ; 1
    addc A,#255 and ONE_MS shr 8 ; 1
    mov  TH0,A                   ; 1
;-----------------------------------------------------------------------
    setb TR0                     ; 1
;-----------------------------------------------------------------------
    SELECT_BANK 0
;-----------------------------------------------------------------------
    djnz R3,.scan_ok
    mov  R3,#10
    mov  KEYS,P1
    setb TEN_MS
;-----------------------------------------------------------------------
.scan_ok:
    pop  ACC
    pop  PSW
;-----------------------------------------------------------------------
    reti
;=======================================================================
;
;=======================================================================
start:
;-----------------------------------------------------------------------
    SELECT_BANK 0
    mov  SP,#STACK
;-----------------------------------------------------------------------
    mov  R3,#1
;-----------------------------------------------------------------------
    mov  TH0,#-1
    mov  TL0,#-1
;-----------------------------------------------------------------------
    mov  TMOD,#01010001b
    setb ET1
    setb ET0
    setb EA
    setb TR0
;-----------------------------------------------------------------------
    SELECT_BANK 1
;-----------------------------------------------------------------------
    call screen_init
;-----------------------------------------------------------------------
mainloop:
;-----------------------------------------------------------------------
    mov  TH1,#0
    mov  TL1,#0
    mov  TX1,#0
    mov  R7,#100
;-----------------------------------------------------------------------
    clr  TEN_MS
    jnb  TEN_MS,$
    nop
    nop
    setb TR1
;-----------------------------------------------------------------------
.loop:
    clr  TEN_MS
    jnb  TEN_MS,$
    djnz R7,.loop
    clr  TR1
;-----------------------------------------------------------------------
    mov  R1,TX1
    mov  R2,TH1
    mov  R3,TL1
    call SUB_BIN24TOBCD4
;-----------------------------------------------------------------------
    setb BLANK_ZERO
    mov  R2,#7
;-----------------------------------------------------------------------
    mov  A,R4
    call show_sym_x
;-----------------------------------------------------------------------
    mov  A,R5
    swap A
    call show_sym_x
;-----------------------------------------------------------------------
    mov  A,R5
    call show_sym_x
;-----------------------------------------------------------------------
    mov  A,R6
    swap A
    call show_sym_x
;-----------------------------------------------------------------------
    mov  A,#DIG_X
    mov  B,R2
    call show_sym  
    dec  R2  
;-----------------------------------------------------------------------
    mov  A,R6
    call show_sym_x
;-----------------------------------------------------------------------
    mov  A,R7
    swap A
    call show_sym_x
;-----------------------------------------------------------------------
    mov  A,R7
    clr  BLANK_ZERO
    call show_sym_x
;-----------------------------------------------------------------------
    jmp  mainloop
;-----------------------------------------------------------------------
show_sym_x:
    mov  B,R2
    dec  R2
    jnb  BLANK_ZERO,.norm
    anl  A,#15
    jnz  .nonzero
    mov  A,#DIG_X
    jmp  show_sym
.nonzero:    
    clr  BLANK_ZERO
.norm:
    call num2led
    jmp  show_sym
;=======================================================================
include "bin24tobcd4.inc"
;=======================================================================
    
 